#!/usr/bin/env python
# -*- coding: utf-8 -*-
##################################################################
#
# Copyright (c) 2023 CICV, Inc. All Rights Reserved
#
##################################################################
"""
@Authors:           zhanghaiwen, yangzihao
@Data:              2024/02/21
@Last Modified:     2024/02/21
@Summary:           The template of custom indicator.
"""

"""
设计思路：

"""

import math
import pandas as pd
import numpy as np
from common import zip_time_pairs, continuous_group, get_status_active_data
from log import logger

"""import functions"""


# custom metric codes
class CustomMetric(object):
    def __init__(self, all_data, case_name):
        self.data = all_data
        self.optimal_dict = self.data.config
        self.status_trigger_dict = self.data.status_trigger_dict
        self.case_name = case_name
        self.markline_df = pd.DataFrame(columns=['start_time', 'end_time', 'start_frame', 'end_frame', 'type'])
        self.graph_list = []

        self.df = pd.DataFrame()
        self.ego_df = pd.DataFrame()
        self.df_ica = pd.DataFrame()

        self.percentage_accel = 0

        self.result = {
            "name": "合理加减速度占比",
            "value": [],
            # "weight": [],
            "tableData": {
                "avg": "",  # 平均值，或指标值
                "max": "",
                "min": ""
            },
            "reportData": {
                "name": "合理加减速度占比（%）",
                # "legend": [], # 如果有多个data，则需要增加data对应的说明，如：["横向加速度", "纵向加速度"]
                "data": [],
                "markLine": [],
                "range": [],
            },
            "statusFlag": {}
        }
        self.run()
        print(f"合理加减速度占比: {self.result['value']}")

    def data_extract(self):
        self.df = self.data.object_df
        self.ego_df = self.data.ego_data
        active_time_ranges = self.status_trigger_dict['ACC']['ACC_active_time']
        self.df_acc = get_status_active_data(active_time_ranges, self.ego_df)
        # self.df_acc = self.ego_df[self.ego_df['ICA_status'] == "LLC_Follow_Line"].copy()  # 数字3对应ICA的Active
        # self.df_acc = self.df[self.df['ACC_status'] == "Shut_off"].copy()  # 数字3对应ICA的Active
        # self.df_acc = self.df[self.df['ACC_status'] == "Active"].copy()  # 数字3对应ICA的Active

        if self.df_acc.empty:
            self.result['statusFlag']['function_ACC'] = False
        else:
            self.result['statusFlag']['function_ACC'] = True
    def data_analyze(self):
        col_list = ['simTime', 'simFrame', 'playerId', 'v', 'accel', 'lon_acc_roc']  # 列表指定了要分析的列  
        # df = self.df_acc[col_list].copy()  # 从原始 DataFrame 中选择列并复制  

        try:  
            df = self.df_acc[col_list].copy()  
        except KeyError as e:  
            print(f"Error: {e}. Column '{col_list}' not found in DataFrame.")  
    # 这里可以添加更多的错误处理逻辑，比如退出程序或者设置默认值等
    
        # 将速度从km/h 转换为 m/s
        df['v_ms'] = df['v'] / 3.6  
    
        # 初始化计数器和阈值  
        count_in_range = 0  
        total_frames = len(df)  
    
        # 遍历DataFrame的每一行  
        for index, row in df.iterrows():  
            v_ms = row['v_ms']  
            accel = row['accel']  
    
            # 根据车速范围设置减速度和加速度的阈值  
            if v_ms > 20:  # 车速大于20km/h  
                decel_threshold = -3.5  
                accel_threshold = 2  
            elif v_ms < 5:  # 车速低于5km/h  
                decel_threshold = 5  
                accel_threshold = 4  # 注意：这里假设是4而不是44，因为44对于加速度来说是一个很大的值  
            else:  # 车速在5到20km/h之间  
                # 对减速度和加速度的阈值进行线性插值  
                # 假设在5km/h时decel_threshold=5, accel_threshold=4  
                # 在20km/h时decel_threshold=-3.5, accel_threshold=2  
                decel_threshold = 5 - (5 + 3.5) * ((v_ms - 5) / (20 - 5))  
                accel_threshold = 4 - (4 - 2) * ((v_ms - 5) / (20 - 5))  
    
            # 检查加速度是否在允许的范围内  
            if accel >= decel_threshold and accel <= accel_threshold:  
                count_in_range += 1  
    
        # 计算百分比  
        percentage_in_range = (count_in_range / total_frames) * 100  
    
        # 存储结果并打印  
        self.result['value'] = [round(percentage_in_range, 3)]  
        print(f"Percentage of frames with acceleration in range: {percentage_in_range}%")  


    def markline_statistic(self):
        pass

    def report_data_statistic(self):
        time_list = self.ego_df['simTime'].values.tolist()
        graph_list = [x for x in self.graph_list if not np.isnan(x)]
        self.result['tableData']['avg'] = f'{np.mean(graph_list):.2f}' if graph_list else 0
        self.result['tableData']['max'] = f'{max(graph_list):.2f}' if graph_list else 0
        self.result['tableData']['min'] = f'{min(graph_list):.2f}' if graph_list else 0

        zip_vs_time = zip_time_pairs(time_list, self.graph_list)
        self.result['reportData']['data'] = zip_vs_time

        # self.markline_statistic()
        # markline_slices = self.markline_df.to_dict('records')
        self.result['reportData']['markLine'] = []

        self.result['reportData']['range'] = [0, 100]

    def run(self):
        # logger.info(f"Custom metric run:[{self.result['name']}].")
        logger.info(f"[case:{self.case_name}] Custom metric:[percentage_accel:{self.result['name']}] evaluate.")

        try:
            self.data_extract()
        except Exception as e:
            logger.error(f"[case:{self.case_name}] Custom metric:{self.result['name']} data extract ERROR!", e)

        try:
            self.data_analyze()
        except Exception as e:
            logger.error(f"[case:{self.case_name}] Custom metric:{self.result['name']} data analyze ERROR!", e)

        try:
            self.report_data_statistic()
        except Exception as e:
            logger.error(f"[case:{self.case_name}] Custom metric:{self.result['name']} report data statistic ERROR!", e)

# if __name__ == "__main__":
#     pass
